<?php
/**
* bookscanからPDFを一括ダウンロード
*
* @note
* - PHP versions 5
* - install
@code
pear install HTTP_Client
php -q bookscan_dl.php
@endcode
*
* @licence LGPLv3 http://sourceforge.jp/projects/opensource/wiki/licenses%2FGNU_Lesser_General_Public_License_version_3.0
* @author dexdev@gmail.com
* @version $Revision: 152 $
*/

defined('DS') or define('DS', DIRECTORY_SEPARATOR);
defined('CUR_DIR') or define('CUR_DIR', dirname(__FILE__));
define('LOGIN_USER_ID', 'YOUR MAIL ADDR');	// ログインID
define('LOGIN_PASSWORD', 'YOUR PASSWORD');				// ログインパスワード
define('DOWNLOAD_TARGET', 'http://system.bookscan.co.jp/bookdetail.php?hash=XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'); // 依頼URL
define('DOWNLOAD_PATH', CUR_DIR . DS . 'download');		// 保存先

mb_internal_encoding('UTF-8');
mb_http_input('pass');
mb_http_output('sjis-win'); // 出力文字コード。Windows: sjis-win, Linux: UTF-8
ini_set("max_execution_time", 24*60*60); // タイムアウト(s) 24時間
ini_set("memory_limit", "1024M"); // メモリ 1GB

// 実行
exit(main($argc, $argv));

/// main
function main($argc, $argv)
{
	$bookscan = new Bookscan();
	
	try{
		$bookscan->login(LOGIN_USER_ID, LOGIN_PASSWORD);
		$bookscan->download(DOWNLOAD_TARGET, DOWNLOAD_PATH);
	}catch(Exception $e){
		fwrite(STDERR, "Error : " . $e->getMessage());
	}
}


/**
* Bookscanクラス
*/
class Bookscan
{
	var $client = null;

	/// コンストラクタ
	function Bookscan($options = array())
	{
		require_once("HTTP/Client.php");
		require_once("simplehtmldom/simple_html_dom.php");
	}

	/// ログイン
	function login($id, $password)
	{
		$login_params = array(
			"next_url" => 'http://system.bookscan.co.jp/mypage.php',
			"email" => $id,
			"password" => $password,
		);

		$login_url = "http://system.bookscan.co.jp/login.php";

		$this->client = new HTTP_Client();
		$this->client->post($login_url, $login_params);
	}
	
	/// ダウンロード
	function download($target_url, $save_path, $options = array()){
		$_options = array(
			'overwrite' => false,	// ダウンロード時に上書きするか
			'tries' => 3,			// リトライ回数
		);
		$_options = array_merge($_options, $options);

		// 保存先作成
		if( !is_dir($save_path) && !mkdir($save_path, 0700, true) ){
			throw new Exception('Error: Failed to create folders... ' . $save_path);
		}

		// 書籍一覧
		$history_url = "http://system.bookscan.co.jp/history.php";
		$this->client->get($target_url);
		$response = $this->client->currentResponse();
		
		if( $response['code'] != 200 ) {
			throw new Exception( sprintf('Error: Failed to download ... code: %d, url: %s', $response['code'], $history_url) );
		}

//		$response = file_get_contents('0');
		$html = str_get_html($response['body']);
		foreach($html->find('.download') as $element){
//			echo $element->outertext(); // <a ...> ～ </a>
//			echo $element->innertext(); // 2010年07月29日14時48分09秒.pdf
//			echo $element->plaintext; // 2010年07月29日14時48分09秒.pdf
			
			if( !preg_match('#\.pdf$#i', $element->plaintext) ) continue;

			$filename = mb_convert_encoding($element->plaintext, mb_http_output(), 'UTF-8');
			$file = $save_path . DS . $filename;

			if( !$_options['overwrite'] && is_file($file) ) {
				fwrite(STDERR, sprintf("Notice: Skip file. %s\n", $filename));
				flush();
				continue;
			}

			if( !$wfp = fopen($file, "wb") ){
				throw new Exception('Error: Failed to create file... ' . $file);
			}
			
			$this->client->get($element->href);
			$data = $this->client->currentResponse();
			if( $response['code'] != 200 ) {
				fclose($wfp);
				unlink($file);
				throw new Exception( sprintf('Error: Failed to download ... code: %d, url: %s', $data['code'], $element->href) );
			}

			fwrite($wfp, $data['body']);
			unset($data);
			$this->client->_responses = array(); // メモリ不足になるため。reset()だと、セッションも切れる
//			$this->client->reset();
			
			fclose($wfp);
			
			fwrite(STDERR, sprintf("Success: Download complete. %s\n", $filename));
			flush();
		}
	}
}
